// This file was generated by qtwaylandscanner
// source file is /scratch/jenkins-BUILDS-eSDK-2024.1_stable-pipeline-build-179_Ultra96Full/build/tmp/work/ultra96_zynqmp-xilinx-linux/qtwayland/5.15.4+gitAUTOINC+f1e6c8764d-r0/git/src/extensions/qt-key-unstable-v1.xml

#include <QtWaylandClient/private/qwayland-qt-key-unstable-v1.h>

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")

namespace QtWayland {

static inline void *wlRegistryBind(struct ::wl_registry *registry, uint32_t name, const struct ::wl_interface *interface, uint32_t version)
{
    const uint32_t bindOpCode = 0;
#if (WAYLAND_VERSION_MAJOR == 1 && WAYLAND_VERSION_MINOR > 10) || WAYLAND_VERSION_MAJOR > 1
    return (void *) wl_proxy_marshal_constructor_versioned((struct wl_proxy *) registry,
        bindOpCode, interface, version, name, interface->name, version, nullptr);
#else
    return (void *) wl_proxy_marshal_constructor((struct wl_proxy *) registry,
        bindOpCode, interface, name, interface->name, version, nullptr);
#endif
}

    zqt_key_v1::zqt_key_v1(struct ::wl_registry *registry, int id, int version)
    {
        init(registry, id, version);
    }

    zqt_key_v1::zqt_key_v1(struct ::zqt_key_v1 *obj)
        : m_zqt_key_v1(obj)
    {
        init_listener();
    }

    zqt_key_v1::zqt_key_v1()
        : m_zqt_key_v1(nullptr)
    {
    }

    zqt_key_v1::~zqt_key_v1()
    {
    }

    void zqt_key_v1::init(struct ::wl_registry *registry, int id, int version)
    {
        m_zqt_key_v1 = static_cast<struct ::zqt_key_v1 *>(wlRegistryBind(registry, id, &zqt_key_v1_interface, version));
        init_listener();
    }

    void zqt_key_v1::init(struct ::zqt_key_v1 *obj)
    {
        m_zqt_key_v1 = obj;
        init_listener();
    }

    zqt_key_v1 *zqt_key_v1::fromObject(struct ::zqt_key_v1 *object)
    {
        if (wl_proxy_get_listener((struct ::wl_proxy *)object) != (void *)&m_zqt_key_v1_listener)
            return nullptr;
        return static_cast<zqt_key_v1 *>(zqt_key_v1_get_user_data(object));
    }

    bool zqt_key_v1::isInitialized() const
    {
        return m_zqt_key_v1 != nullptr;
    }

    const struct wl_interface *zqt_key_v1::interface()
    {
        return &::zqt_key_v1_interface;
    }

    void zqt_key_v1::zqt_key_v1_key(struct ::wl_surface *, uint32_t , uint32_t , uint32_t , uint32_t , uint32_t , uint32_t , uint32_t , const QString &, uint32_t , uint32_t )
    {
    }

    void zqt_key_v1::handle_key(
        void *data,
        struct ::zqt_key_v1 *object,
        struct ::wl_surface *surface,
        uint32_t time,
        uint32_t type,
        uint32_t key,
        uint32_t modifiers,
        uint32_t nativeScanCode,
        uint32_t nativeVirtualKey,
        uint32_t nativeModifiers,
        const char *text,
        uint32_t autorepeat,
        uint32_t count)
    {
        Q_UNUSED(object);
        static_cast<zqt_key_v1 *>(data)->zqt_key_v1_key(
            surface,
            time,
            type,
            key,
            modifiers,
            nativeScanCode,
            nativeVirtualKey,
            nativeModifiers,
            QString::fromUtf8(text),
            autorepeat,
            count);
    }

    const struct zqt_key_v1_listener zqt_key_v1::m_zqt_key_v1_listener = {
        zqt_key_v1::handle_key,
    };

    void zqt_key_v1::init_listener()
    {
        zqt_key_v1_add_listener(m_zqt_key_v1, &m_zqt_key_v1_listener, this);
    }
}

QT_WARNING_POP
QT_END_NAMESPACE
